///////////////////////////////////////////////////////////////////////////////
// DeSerialize - Read serialization of Java objects
// Copyright (C) 2004 C. Brian Jones <cbj@gnu.org>
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
///////////////////////////////////////////////////////////////////////////////

package com.oryxsoft.test.serialization;

import gnu.getopt.Getopt;

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.LineNumberReader;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.ObjectInputStream;
import java.io.FileNotFoundException;

/**
 * This program can be used to read serialized objects from 
 * individual files for each class.  The files are read 
 * through the normal serialization process for Java.
 *
 * @author C. Brian Jones &lt;<a href="mailto:cbj@gnu.org">cbj@gnu.org</a>&gt;
 */
public class Deserialize
{
  private static String _inputDirectory = ".";
  
  public static void main (String[] argv)
  {
    if (argv.length < 1) 
      help ();

    Getopt g = new Getopt ("Serialize", argv, "d:x");

    int c;
    String arg;
    while ((c = g.getopt ()) != -1)
    {
      switch (c)
      {
        case 'd':
          _inputDirectory = g.getOptarg ();
          break;
        case '?':
          help ();
          break;
        default:
          help ();
          break;
      }
    }
    
    Deserialize ser = new Deserialize ();
    
    for (int i = g.getOptind(); i < argv.length ; i++)
    {
      File f = new File (argv[i]);
      if (f.exists ())
        ser.readFile (f);
      else
        ser.readClass (argv[i]);
    }

    // In case AWT threads do not die appropriately
    System.exit (0);
  }
  
  private void readFile (File f)
  {
    try
    {
      LineNumberReader reader = new LineNumberReader (new BufferedReader (new FileReader (f)));
      String cls;
      while ((cls = reader.readLine ()) != null)
        readClass (cls);
      reader.close ();
    }
    catch (Throwable t) 
    {
      System.err.println (t.getMessage ());
      System.exit (1);
    }
  }

  private Class _serial = null;
  private ObjectInputStream _objin = null;  
  private File _file = null;
  
  private void readClass (String clsName)
  {
    System.out.print (clsName);
    
    try
    {
      String filePrefix = clsName.replace ('.', File.separatorChar);
      StringBuffer buf = new StringBuffer (_inputDirectory);
      buf.append (File.separator);
      if (filePrefix.indexOf (File.separator) != -1)
        buf.append (filePrefix.substring (0, filePrefix.lastIndexOf (File.separator)));
      else
        buf.append (filePrefix);
      String dirPrefix = buf.toString ();
      File dir = new File (dirPrefix);
    
        _file = new File (_inputDirectory, filePrefix + Constants.BINARY_TYPE);        
	if (! _file.exists())
	  throw new FileNotFoundException(_file.getAbsolutePath() + " not found");
        _objin = new ObjectInputStream (new BufferedInputStream (new FileInputStream (_file)));
    }
    catch (Throwable t)
    {
      System.out.println (": error, " + t.getClass().getName() + ": " + t.getMessage ());
//       t.printStackTrace ();
      close ();
      return;
    }
      
    Object obj = null;
    boolean readObj = false;
    try
    {
        obj = _objin.readObject ();
	readObj = true;
    }
    catch (ClassNotFoundException e)
    {
      System.out.println (": fail, class not found");
      close ();
      return;
    }
    catch (Throwable t)
    {
      System.out.println (": error, " + t.getMessage ());
//      t.printStackTrace ();
      close ();
      return;
    }
    System.out.println (": ok");

    close ();
  }

  private void close ()
  {    
    if (_objin != null)
    {
      try
      {
        _objin.close ();
      }
      catch (Throwable t) { }    
    }
  }

  private static void help ()
  {
    System.err.println ("DeSerialize version " + Constants.VERSION + ", (c) 2004 C. Brian Jones");
    System.err.println ("Usage: deserialize [-d] [FILE|CLASS]");
    System.err.println ("  -d <directory>    input directory");
    System.exit (1);
  }
}

